@@ -16,6 +16,8 @@ __all__ = [
     'lb',
     'tree2array',
     'tree2rec',
+    'array2tree',
+    'array2root',
     'fill_array',
 ]
 
@@ -87,9 +89,10 @@ def lst(filename, treename=None):
 def root2array(filenames,
                treename=None,
                branches=None,
-               entries=None,
-               offset=0,
-               selection=None):
+               selection=None,
+               start=None,
+               stop=None,
+               step=None):
     """
     Convert trees in ROOT files into a numpy structured array.
     Refer to the type conversion table :ref:`here <conversion_table>`.
@@ -107,15 +110,14 @@ def root2array(filenames,
         If None or empty then include all branches than can be converted in the
         first tree.
         If branches contains duplicate branches, only the first one is used.
-    entries : int, optional (default=None)
-        Maximum number of entries that will be converted from the chained
-        trees. If None then convert all entries. If a selection is applied then
-        fewer entries may be converted.
-    offset : int, optional (default=0):
-        Offset from the beginning of the chained trees where conversion will
-        begin.
     selection : str, optional (default=None)
-        Only include entries passing a cut expression.
+        Only include entries fulfilling this condition.
+    start, stop, step: int, optional (default=None)
+        The meaning of the ``start``, ``stop`` and ``step``
+        parameters is the same as for Python slices.
+        If a range is supplied (by setting some of the
+        ``start``, ``stop`` or ``step`` parameters), only the entries in that
+        range and fulfilling the ``selection`` condition (if defined) are used.
 
     Examples
     --------
@@ -128,7 +130,15 @@ def root2array(filenames,
     Read all branches starting from entry 5 and include 10 entries or up to the
     end of the file::
 
-        root2array('a.root', 'mytree', entries=10, offset=5)
+        root2array('a.root', 'mytree', start=5, stop=11)
+
+    Read all branches in reverse order::
+
+        root2array('a.root', 'mytree', step=-1)
+
+    Read every second entry::
+
+        root2array('a.root', 'mytree', step=2)
 
     Read all branches from the tree named ``mytree`` in ``a*.root``::
 
@@ -175,15 +185,16 @@ def root2array(filenames,
             treename = trees[0]
 
     return _librootnumpy.root2array_fromFname(
-        matched_filenames, treename, branches, entries, offset, selection)
+        matched_filenames, treename, branches, selection, start, stop, step)
 
 
 def root2rec(filenames,
              treename=None,
              branches=None,
-             entries=None,
-             offset=0,
-             selection=None):
+             selection=None,
+             start=None,
+             stop=None,
+             step=None):
     """
     View the result of :func:`root2array` as a record array.
 
@@ -197,15 +208,16 @@ def root2rec(filenames,
     --------
     root2array
     """
-    return root2array(filenames, treename, branches,
-                      entries, offset, selection).view(np.recarray)
+    return root2array(filenames, treename, branches, selection,
+                      start, stop, step).view(np.recarray)
 
 
 def tree2array(tree,
                branches=None,
-               entries=None,
-               offset=0,
                selection=None,
+               start=None,
+               stop=None,
+               step=None,
                include_weight=False,
                weight_name='weight',
                weight_dtype='f4'):
@@ -222,15 +234,14 @@ def tree2array(tree,
         If None or empty then include all branches than can be converted in the
         first tree.
         If branches contains duplicate branches, only the first one is used.
-    entries : int, optional (default=None)
-        Maximum number of entries that will be converted from the chained
-        trees. If None then convert all entries. If a selection is applied then
-        fewer entries may be converted.
-    offset : int, optional (default=0):
-        Offset from the beginning of the chained trees where conversion will
-        begin.
     selection : str, optional (default=None)
-        Only include entries passing a cut expression.
+        Only include entries fulfilling this condition.
+    start, stop, step: int, optional (default=None)
+        The meaning of the ``start``, ``stop`` and ``step``
+        parameters is the same as for Python slices.
+        If a range is supplied (by setting some of the
+        ``start``, ``stop`` or ``step`` parameters), only the entries in that
+        range and fulfilling the ``selection`` condition (if defined) are used.
     include_weight : bool, optional (default=False)
         Include a column containing the tree weight.
     weight_name : str, optional (default='weight')
@@ -253,7 +264,7 @@ def tree2array(tree,
         #return _librootnumpy.root2array_from_capsule(o, branches)
     cobj = ROOT.AsCObject(tree)
     arr = _librootnumpy.root2array_fromCObj(
-        cobj, branches, entries, offset, selection)
+        cobj, branches, selection, start, stop, step)
     if include_weight:
         arr = _add_weight_field(arr, tree, weight_name, weight_dtype)
     return arr
@@ -261,9 +272,10 @@ def tree2array(tree,
 
 def tree2rec(tree,
              branches=None,
-             entries=None,
-             offset=0,
              selection=None,
+             start=None,
+             stop=None,
+             step=None,
              include_weight=False,
              weight_name='weight',
              weight_dtype='f4'):
@@ -282,15 +294,85 @@ def tree2rec(tree,
 
     """
     return tree2array(tree,
-                      branches,
-                      entries,
-                      offset,
-                      selection,
+                      branches=branches,
+                      selection=selection,
+                      start=start,
+                      stop=stop,
+                      step=step,
                       include_weight=include_weight,
                       weight_name=weight_name,
                       weight_dtype=weight_dtype).view(np.recarray)
 
 
+def array2tree(arr, name='tree', tree=None):
+    """
+    Convert a numpy structured array into a ROOT TTree.
+
+    .. warning::
+       This function is experimental. Please report problems.
+       Not all data types are supported (``np.object`` and ``np.float16``).
+
+    Parameters
+    ----------
+    arr : array
+        A numpy structured array
+    name : str (optional, default='tree')
+        Name of the created ROOT TTree if ``tree`` is None.
+    tree : existing ROOT TTree (optional, default=None)
+        Any branch with the same name as a field in the
+        numpy array will be extended as long as the types are compatible,
+        otherwise a TypeError is raised. New branches will be created
+        and filled for all new fields.
+
+    Returns
+    -------
+    root_tree : a ROOT TTree
+
+    See Also
+    --------
+    array2root
+
+    """
+    import ROOT
+    if tree is not None:
+        if not isinstance(tree, ROOT.TTree):
+            raise TypeError("tree must be a ROOT.TTree")
+        incobj = ROOT.AsCObject(tree)
+    else:
+        incobj = None
+    cobj = _librootnumpy.array2tree_toCObj(arr, name=name, tree=incobj)
+    return ROOT.BindObject(cobj, 'TTree')
+
+
+def array2root(arr, filename, treename='tree', mode='update'):
+    """
+    Convert a numpy structured array into a ROOT TTree and save directly in a
+    ROOT TFile.
+
+    .. warning::
+       This function is experimental. Please report problems.
+       Not all data types are supported (``np.object`` and ``np.float16``).
+
+    Parameters
+    ----------
+    arr : array
+        A numpy structured array
+    filename : str
+        Name of the output ROOT TFile. A new file will be created if it
+        doesn't already exist.
+    treename : str (optional, default='tree')
+        Name of the created ROOT TTree.
+    mode : str (optional, default='update')
+        Mode used to open the ROOT TFile ('update' or 'recreate').
+
+    See Also
+    --------
+    array2tree
+
+    """
+    _librootnumpy.array2root(arr, filename, treename, mode)
+
+
 def fill_array(hist, array, weights=None):
     """Fill a ROOT histogram with a NumPy array.
 
