@@ -1,16 +1,17 @@
+import os
 from os.path import dirname, join
+import tempfile
+
 import numpy as np
 from numpy.testing import assert_array_equal
-from root_numpy import *
+
 from ROOT import TChain, TFile, TTree, TH1D, TH2D, TH3D
-try:
-    from collections import OrderedDict
-except ImportError:
-    # Fall back on drop-in
-    from root_numpy.OrderedDict import OrderedDict
+
+from root_numpy import *
+from root_numpy.extern.ordereddict import OrderedDict
+
 import unittest
-from nose.tools import *
-from nose.tools import assert_equal, assert_almost_equal
+from nose.tools import raises, assert_equal, assert_almost_equal
 
 
 class TestRootNumpy(unittest.TestCase):
@@ -46,9 +47,9 @@ class TestRootNumpy(unittest.TestCase):
     def test_lst(self):
         structure = lst(self.ld('single1.root'))
         expected = OrderedDict([
-            ('n_int', [('n_int', 'Int_t')]),
-            ('f_float', [('f_float', 'Float_t')]),
-            ('d_double', [('d_double', 'Double_t')])])
+            ('n_int', [('n_int', 'int')]),
+            ('f_float', [('f_float', 'float')]),
+            ('d_double', [('d_double', 'double')])])
         assert_equal(structure, expected)
 
     def test_single(self):
@@ -166,6 +167,7 @@ class TestRootNumpy(unittest.TestCase):
             a.dtype,
             [('v_i', 'O'),
              ('v_f', 'O'),
+             ('v_F', 'O'),
              ('v_d', 'O'),
              ('v_l', 'O'),
              ('v_c', 'O'),
@@ -173,6 +175,7 @@ class TestRootNumpy(unittest.TestCase):
 
         assert_equal(a.v_i[1].dtype, np.int32)
         assert_equal(a.v_f[1].dtype, np.float32)
+        assert_equal(a.v_F[1].dtype, np.float32)
         assert_equal(a.v_d[1].dtype, np.float64)
         assert_equal(a.v_l[1].dtype, np.int64)
         assert_equal(a.v_c[1].dtype, np.int8)
@@ -189,16 +192,21 @@ class TestRootNumpy(unittest.TestCase):
         assert_equal(a.v_f[-1][0], 198.0)
         assert_equal(a.v_f[-1][-1], 206.0)
 
-    def test_offset_entries(self):
-        a = root2rec(self.ld('single1.root'), entries=10)
+        assert_equal(a.v_F[1][0], 2.0)
+        assert_equal(a.v_F[2][1], 5.0)
+        assert_equal(a.v_F[-1][0], 198.0)
+        assert_equal(a.v_F[-1][-1], 206.0)
+
+    def test_slice(self):
+        a = root2rec(self.ld('single1.root'), stop=10)
         assert_equal(len(a), 10)
         assert_equal(a.n_int[-1], 10)
 
-        a = root2rec(self.ld('single1.root'), entries=10, offset=1)
+        a = root2rec(self.ld('single1.root'), stop=11, start=1)
         assert_equal(len(a), 10)
         assert_equal(a.n_int[-1], 11)
 
-        a = root2rec(self.ld('single1.root'), entries=10, offset=95)
+        a = root2rec(self.ld('single1.root'), stop=105, start=95)
         assert_equal(len(a), 5)
         assert_equal(a.n_int[-1], 100)
 
@@ -272,6 +280,81 @@ class TestRootNumpy(unittest.TestCase):
         assert_almost_equal(stretched.scalar[10],4)
         assert_almost_equal(stretched.scalar[9],3)
 
+    def test_blockwise_inner_join(self):
+        test_data = np.array([
+            (1.0,np.array([11,12,13]),np.array([1,0,1]),0,np.array([1,2,3])),
+            (2.0,np.array([21,22,23]),np.array([-1,2,-1]),1,np.array([31,32,33]))
+            ],
+            dtype=[('sl',np.float),('al','O'),('fk','O'),
+                   ('s_fk',np.int),('ar','O')])
+        #vetor join
+        a1 = blockwise_inner_join(test_data, ['sl','al'], test_data['fk'], ['ar'])
+
+        exp1 = np.array([(1.0, 11, 2, 1),
+                      (1.0, 12, 1, 0),
+                      (1.0, 13, 2, 1),
+                      (2.0, 22, 33, 2)],
+                    dtype=[('sl', '<f8'), ('al', '<i8'),
+                           ('ar', '<i8'), ('fk1', '<i8')])
+        assert_array_equal(a1, exp1, verbose=True)
+
+        #vector join with force repeat
+        a2 = blockwise_inner_join(test_data, ['sl','al'], test_data['fk'], ['ar'],
+                                  force_repeat=['al'])
+        exp2 = np.array([
+                        (1.0, np.array([11, 12, 13]), 2, 1),
+                        (1.0, np.array([11, 12, 13]), 1, 0),
+                        (1.0, np.array([11, 12, 13]), 2, 1),
+                        (2.0, np.array([21, 22, 23]), 33, 2)],
+                        dtype=[('sl', '<f8'), ('al', '|O8'),
+                               ('ar', '<i8'), ('fk1', '<i8')])
+        assert_equal(str(a2), str(exp2))#numpy testing doesn't like subarray
+        assert_equal(a2.dtype, exp2.dtype)
+
+        #scalar join
+        a3 = blockwise_inner_join(test_data, ['sl','al'], test_data['s_fk'], ['ar'])
+        exp3 = np.array([
+                    (1.0, [11, 12, 13], 1, 0),
+                    (2.0, [21, 22, 23], 32, 1)],
+                    dtype=[('sl', '<f8'), ('al', '|O8'),
+                           ('ar', '<i8'), ('fk1', '<i8')])
+        assert_equal(str(a3), str(exp3))#numpy testing doesn't like subarray
+        assert_equal(a3.dtype, exp3.dtype)
+
+    def test_struct(self):
+        assert_array_equal(root2rec(self.ld('structbranches.root')),
+            np.array([(10, 15.5, 20, 781.2)],
+                     dtype=[('branch1_intleaf', '<i4'),
+                            ('branch1_floatleaf', '<f4'),
+                            ('branch2_intleaf', '<i4'),
+                            ('branch2_floatleaf', '<f4')]))
+
+    def test_array2tree(self):
+        a = np.array([(12345, 2., 2.1, True),
+                      (3, 4., 4.2, False),],
+            dtype=[
+                ('x', np.int32),
+                ('y', np.float32),
+                ('z', np.float64),
+                ('w', np.bool)])
+        tree = array2tree(a)
+        a_conv = tree2array(tree)
+        assert_array_equal(a, a_conv)
+        tree.Delete()
+
+    def test_array2root(self):
+        a = np.array([(12345, 2., 2.1, True),
+                      (3, 4., 4.2, False),],
+            dtype=[
+                ('x', np.int32),
+                ('y', np.float32),
+                ('z', np.float64),
+                ('w', np.bool)])
+        tmp_fd, tmp_path = tempfile.mkstemp(suffix='.root')
+        array2root(a, tmp_path, mode='recreate')
+        os.close(tmp_fd)
+        os.remove(tmp_path)
+
 
 if __name__ == '__main__':
     import nose
